"use strict";

exports.__esModule = true;
exports.default = createElement;

var _host = _interopRequireDefault(require("./vdom/host"));

var _element = _interopRequireDefault(require("./vdom/element"));

var _flattenChildren = _interopRequireDefault(require("./vdom/flattenChildren"));

var _error = require("./error");

var _types = require("./types");

var _warning = _interopRequireDefault(require("./warning"));

var _validateChildKeys = _interopRequireDefault(require("./validateChildKeys"));

var _getRenderErrorInfo = _interopRequireDefault(require("./getRenderErrorInfo"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var RESERVED_PROPS = {
  key: true,
  ref: true
};

function createElement(type, config, children) {
  if (type == null) {
    if (process.env.NODE_ENV !== 'production') {
      throw new Error('createElement: type should not be null or undefined.' + (0, _getRenderErrorInfo.default)());
    } else {
      (0, _error.invokeMinifiedError)(0);
    }
  } // Reserved names are extracted


  var props = {};
  var propName;
  var key = null;
  var ref = null;
  var ownerComponent = _host.default.owner;

  if (config != null) {
    var hasReservedProps = false;

    if (config.ref != null) {
      hasReservedProps = true;
      ref = config.ref;

      if (process.env.NODE_ENV !== 'production') {
        if ((0, _types.isString)(ref) && !ownerComponent) {
          (0, _warning.default)('createElement: adding a string ref "' + ref + '" outside the render method.');
        }
      }
    }

    if (config.key != null) {
      hasReservedProps = true;
      key = '' + config.key;
    } // If no reserved props, assign config to props for better performance


    if (hasReservedProps) {
      for (propName in config) {
        // Extract reserved props
        if (!RESERVED_PROPS[propName]) {
          props[propName] = config[propName];
        }
      }
    } else {
      props = config;
    }
  } // Children arguments can be more than one


  var childrenLength = arguments.length - 2;

  if (childrenLength > 0) {
    if (childrenLength === 1 && !(0, _types.isArray)(children)) {
      props.children = children;
    } else {
      var childArray = children;

      if (childrenLength > 1) {
        childArray = new Array(childrenLength);

        for (var i = 0; i < childrenLength; i++) {
          childArray[i] = arguments[i + 2];
        }
      }

      props.children = (0, _flattenChildren.default)(childArray);
    }
  } // Resolve default props


  if (type && type.defaultProps) {
    var defaultProps = type.defaultProps;

    for (propName in defaultProps) {
      if (props[propName] === undefined) {
        props[propName] = defaultProps[propName];
      }
    }
  }

  if (process.env.NODE_ENV !== 'production') {
    for (var _i = 2; _i < arguments.length; _i++) {
      (0, _validateChildKeys.default)(arguments[_i], type);
    }
  }

  return new _element.default(type, key, ref, props, ownerComponent);
}